import _classPrivateFieldGet from "@babel/runtime/helpers/esm/classPrivateFieldGet";
import _classPrivateFieldSet from "@babel/runtime/helpers/esm/classPrivateFieldSet";

function _classPrivateFieldInitSpec(obj, privateMap, value) { _checkPrivateRedeclaration(obj, privateMap); privateMap.set(obj, value); }

function _checkPrivateRedeclaration(obj, privateCollection) { if (privateCollection.has(obj)) { throw new TypeError("Cannot initialize the same private elements twice on an object"); } }

import { BuilderLink } from "../../core/BuilderLink";
import { isString } from "../../validator";
import { PathLocationComposer } from "../composers/PathLocationComposer";
import { PathRouteRecord } from "../RouteRecord";
import { MatchingRouteRecord } from "../RouteRecord/MatchingRouteRecord";
import { isRootPath } from "./isRootPath";

var _link = /*#__PURE__*/new WeakMap();

var _routes = /*#__PURE__*/new WeakMap();

var _wildcardRoute = /*#__PURE__*/new WeakMap();

var _exactRoute = /*#__PURE__*/new WeakMap();

/**
 * Builder to define routes using pathname.
 */
export class PathRouteBuilder {
  static init() {
    return new PathRouteBuilder(new BuilderLink({
      composer: new PathLocationComposer()
    }));
  }

  constructor(link) {
    _classPrivateFieldInitSpec(this, _link, {
      writable: true,
      value: void 0
    });

    _classPrivateFieldInitSpec(this, _routes, {
      writable: true,
      value: Object.create(null)
    });

    _classPrivateFieldInitSpec(this, _wildcardRoute, {
      writable: true,
      value: undefined
    });

    _classPrivateFieldInitSpec(this, _exactRoute, {
      writable: true,
      value: undefined
    });

    _classPrivateFieldSet(this, _link, link);

    link.register(this, (value, remainingLocation) => {
      if (value === undefined) {
        // this is for exact route, so we check whether the route is exact
        if (!isRootPath(remainingLocation)) {
          return undefined;
        }

        const exactRoute = _classPrivateFieldGet(this, _exactRoute);

        return exactRoute && {
          type: "normal",
          value: exactRoute,
          link: exactRoute.getAttachedBuilderLink()
        };
      }

      const route = _classPrivateFieldGet(this, _routes)[value];

      if (route !== undefined) {
        return {
          type: "normal",
          value: route,
          link: route.getAttachedBuilderLink()
        };
      }

      const wildcardRoute = _classPrivateFieldGet(this, _wildcardRoute);

      if (wildcardRoute !== undefined) {
        return {
          type: "matching",
          value: wildcardRoute.route,
          link: wildcardRoute.route.getAttachedBuilderLink(),
          matchKey: wildcardRoute.matchKey,
          matchValue: value
        };
      }

      return undefined;
    });
  }
  /**
   * Add multiple routes at once and return a new Path route builder.
   */


  routes(defs) {
    const result = new PathRouteBuilder(_classPrivateFieldGet(this, _link).inherit());

    const routes = _classPrivateFieldGet(result, _routes);

    Object.assign(routes, _classPrivateFieldGet(this, _routes));

    for (const key of Object.getOwnPropertyNames(defs)) {
      routes[key] = new PathRouteRecord(result, key, defs[key].action);
    }

    _classPrivateFieldSet(result, _wildcardRoute, _classPrivateFieldGet(this, _wildcardRoute));

    _classPrivateFieldSet(result, _exactRoute, _classPrivateFieldGet(this, _exactRoute));

    return result;
  }
  /**
   * Add a route and return a new Path route builder.
   */


  route(key, callback) {
    const result = new PathRouteBuilder(_classPrivateFieldGet(this, _link).inherit());

    const routes = _classPrivateFieldGet(result, _routes);

    Object.assign(routes, _classPrivateFieldGet(this, _routes)); // collect information of route by calling callback.

    let action;
    let attachedBuilder;

    if (callback) {
      const intr = {
        action(a) {
          action = a;
          return this;
        },

        attach(builder) {
          attachedBuilder = builder; // eslint-disable-next-line @typescript-eslint/no-explicit-any

          return builder;
        }

      };
      callback(intr);
    }

    const record = new PathRouteRecord(result, key, action);

    if (attachedBuilder !== undefined) {
      record.attach(attachedBuilder);
    }

    routes[key] = record;

    _classPrivateFieldSet(result, _wildcardRoute, _classPrivateFieldGet(this, _wildcardRoute));

    _classPrivateFieldSet(result, _exactRoute, _classPrivateFieldGet(this, _exactRoute));

    return result;
  }
  /**
   * Add a catch-all route and return a new Path route builder.
   */


  any(key, routeDefinition) {
    const result = new PathRouteBuilder(_classPrivateFieldGet(this, _link).inherit());

    _classPrivateFieldSet(result, _routes, _classPrivateFieldGet(this, _routes));

    _classPrivateFieldSet(result, _wildcardRoute, {
      matchKey: key,
      route: new MatchingRouteRecord(result, // TypeScript requires this `as` but this should be true because Key extends string.
      key, isString, routeDefinition === null || routeDefinition === void 0 ? void 0 : routeDefinition.action)
    });

    _classPrivateFieldSet(result, _exactRoute, _classPrivateFieldGet(this, _exactRoute));

    return result;
  }
  /**
   * Add an exact route and return a new Path route builder.
   */


  exact(routeDefinition) {
    const result = new PathRouteBuilder(_classPrivateFieldGet(this, _link).inherit());

    _classPrivateFieldSet(result, _routes, _classPrivateFieldGet(this, _routes));

    _classPrivateFieldSet(result, _wildcardRoute, _classPrivateFieldGet(this, _wildcardRoute));

    _classPrivateFieldSet(result, _exactRoute, new PathRouteRecord(result, undefined, routeDefinition.action));

    return result;
  }

  getRoutes() {
    const routes = _classPrivateFieldGet(this, _routes);

    return routes;
  }
  /**
   * Shorthand for 'getRoutes()'
   */


  get _() {
    return this.getRoutes();
  }
  /**
   * Route record of the any route.
   */


  get anyRoute() {
    var _classPrivateFieldGet2;

    return (_classPrivateFieldGet2 = _classPrivateFieldGet(this, _wildcardRoute)) === null || _classPrivateFieldGet2 === void 0 ? void 0 : _classPrivateFieldGet2.route;
  }
  /**
   * Route record of the exact route.
   */


  get exactRoute() {
    return _classPrivateFieldGet(this, _exactRoute);
  }

  getBuilderLink() {
    return _classPrivateFieldGet(this, _link);
  }

}