import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _classPrivateFieldSet from "@babel/runtime/helpers/esm/classPrivateFieldSet";
import _classPrivateFieldGet from "@babel/runtime/helpers/esm/classPrivateFieldGet";

function _classPrivateFieldInitSpec(obj, privateMap, value) { _checkPrivateRedeclaration(obj, privateMap); privateMap.set(obj, value); }

function _checkPrivateRedeclaration(obj, privateCollection) { if (privateCollection.has(obj)) { throw new TypeError("Cannot initialize the same private elements twice on an object"); } }

import { RouteResolver } from "../RouteResolver";
import { createSegmentDecomposer } from "../RouteResolver/resolveChain";

var _state = /*#__PURE__*/new WeakMap();

/**
 * Link between parent and child builders.
 */
export class BuilderLink {
  /**
   * Registered current builder.
   */
  constructor(options) {
    _defineProperty(this, "composer", void 0);

    _defineProperty(this, "resolver", void 0);

    _classPrivateFieldInitSpec(this, _state, {
      writable: true,
      value: {
        state: "unattached"
      }
    });

    _defineProperty(this, "currentBuilder", undefined);

    _defineProperty(this, "segmentDecomposer", void 0);

    this.composer = options.composer;
    this.resolver = new RouteResolver(this);
  }
  /**
   * Attach this link to a parent.
   */


  attachToParent(parentLink, segmentGetter) {
    if (_classPrivateFieldGet(this, _state).state !== "unattached") {
      throw new Error("A builder cannot be attached more than once.");
    }

    _classPrivateFieldSet(this, _state, {
      state: "attached",
      parentLink,
      segmentGetter
    });

    this.resolver = parentLink.resolver;
  }
  /**
   * Collect pairs of link and segment between its parent.
   */


  collectUpToTop() {
    const result = []; // eslint-disable-next-line @typescript-eslint/no-this-alias

    let current = this;
    let parent = current.getParentLinkAndSegmentGetter();

    while (parent !== undefined) {
      result.push({
        link: parent[1],
        segmentGetter: parent[0]
      });
      current = parent[1];
      parent = current.getParentLinkAndSegmentGetter();
    }

    return result;
  }

  composeFromTop(defaultRoot, match) {
    const links = this.collectUpToTop().reverse();
    return links.reduce((loc, {
      link,
      segmentGetter: segment
    }) => {
      return link.composer.compose(loc, segment(match));
    }, defaultRoot);
  }
  /**
   * Returns the topmost and uninherited link.
   */


  getAttachmentRoot() {
    const ls = this.collectUpToTop();
    const top = ls.length === 0 ? this : ls[ls.length - 1].link;
    return top;
  }

  getParentLinkAndSegmentGetter() {
    const state = _classPrivateFieldGet(this, _state);

    if (state.state === "attached") {
      return [state.segmentGetter, state.parentLink];
    }

    return undefined;
  }

  getBuilderLink() {
    return this;
  }

  register(builder, resolveSegment) {
    this.currentBuilder = builder;
    this.segmentDecomposer = createSegmentDecomposer(this, resolveSegment);
  }
  /**
   * Create a new BuilderLink which inherits current link.
   */


  inherit() {
    return this;
  }

}