import _extends from "@babel/runtime/helpers/esm/extends";
import React, { forwardRef, useContext, useMemo } from "react";
import { getRouteRecordLocation } from "../../builder/RouteRecord/getRouteRecordLocation";
import { locationToURL } from "../../util/locationToURL";
import { RouteContext } from "../contexts/RouteContext";
import { useNavigate } from "../hooks/useNavigate";
import { getNavigationBaseLocation } from "../logic/getNavigationBaseLocation";

/**
 * Renders an <a> element to given route.
 */
const RawLink = ({
  route,
  match,
  ...props
}, ref) => {
  const parentRoute = useContext(RouteContext);
  const navigate = useNavigate();
  const href = useMemo(() => {
    const baseLocation = getNavigationBaseLocation(parentRoute, route);
    const location = getRouteRecordLocation(route, match || {}, baseLocation);
    return locationToURL(location);
  }, [route, match, parentRoute]);
  return /*#__PURE__*/React.createElement("a", _extends({
    ref: ref,
    href: href,
    onClick: e => {
      if (!isModifiedEvent(e)) {
        e.preventDefault(); // eslint-disable-next-line @typescript-eslint/no-explicit-any

        navigate(route, match);
      }
    }
  }, props));
};

export const Link = /*#__PURE__*/forwardRef(RawLink);

function isModifiedEvent(event) {
  return event.metaKey || event.altKey || event.ctrlKey || event.shiftKey;
}