import { useContext, useMemo } from "react";
import { getRouteRecordLocation } from "../../builder/RouteRecord/getRouteRecordLocation";
import { HistoryContext } from "../contexts/HistoryContext";
import { RouteContext } from "../contexts/RouteContext";
import { getNavigationBaseLocation } from "../logic/getNavigationBaseLocation";
const cache = new WeakMap();
/**
 * Returns the navigate function.
 */

export const useNavigate = () => {
  const historyContextObject = useContext(HistoryContext);
  const parentRoute = useContext(RouteContext);

  if (historyContextObject === undefined) {
    throw new Error("No history found in the context. Please make sure you have placed RoconRoot above.");
  }

  const {
    history
  } = historyContextObject;
  const navigate = useMemo(() => {
    // create navigate function.
    const push = (route, match) => {
      const baseLocation = getNavigationBaseLocation(parentRoute, route);
      history.push(getRouteRecordLocation(route, match || {}, baseLocation));
    };

    const replace = (route, match) => {
      const baseLocation = getNavigationBaseLocation(parentRoute, route);
      history.replace(getRouteRecordLocation(route, match || {}, baseLocation));
    };

    return Object.assign(push, {
      push,
      replace
    });
  }, [history, parentRoute]);
  return navigate;
};