import React, { useContext, useMemo } from "react";
import { RoutePathResolver } from "../../builder/RoutePathResolver";
import { RouteContext } from "../contexts/RouteContext";
import { LocationNotFoundError } from "../errors/LocationNotFoundError";
import { useLocation } from "./useLocation";
/**
 * Receives a route builder and returns a corresponding rendered route.
 */

export const useRoutes = builder => {
  var _parentRoute$nextLoca;

  const parentRoute = useContext(RouteContext);
  const location = useLocation();
  const locationToResolve = (_parentRoute$nextLoca = parentRoute === null || parentRoute === void 0 ? void 0 : parentRoute.nextLocation) !== null && _parentRoute$nextLoca !== void 0 ? _parentRoute$nextLoca : location;
  const resolver = useMemo(() => RoutePathResolver.getFromBuilder(builder), [builder]);
  const {
    routeContextValue,
    result
  } = useMemo(() => {
    const resolved = resolver.resolve(locationToResolve)[0];

    if (resolved === undefined) {
      throw new LocationNotFoundError("Current location could not be resolved.");
    }

    const ancestorRoutes = parentRoute ? parentRoute.ancestorRoutes.concat() : [];
    ancestorRoutes.push({
      link: builder.getBuilderLink().getAttachmentRoot(),
      location: (parentRoute === null || parentRoute === void 0 ? void 0 : parentRoute.routeLocation) || {
        pathname: "/",
        state: null
      }
    }); // const result = resolved.route.action(resolved.match as never);

    const result = /*#__PURE__*/React.createElement(resolved.route.action, resolved.match);
    const routeContextValue = {
      route: resolved.route,
      ancestorRoutes,
      routeLocation: resolved.currentLocation,
      nextLocation: resolved.remainingLocation
    };
    return {
      result,
      routeContextValue
    };
  }, [parentRoute, resolver, locationToResolve]);
  return /*#__PURE__*/React.createElement(RouteContext.Provider, {
    value: routeContextValue
  }, result);
};