"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useRoutes = void 0;

var _react = _interopRequireWildcard(require("react"));

var _RoutePathResolver = require("../../builder/RoutePathResolver");

var _RouteContext = require("../contexts/RouteContext");

var _LocationNotFoundError = require("../errors/LocationNotFoundError");

var _useLocation = require("./useLocation");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Receives a route builder and returns a corresponding rendered route.
 */
const useRoutes = builder => {
  var _parentRoute$nextLoca;

  const parentRoute = (0, _react.useContext)(_RouteContext.RouteContext);
  const location = (0, _useLocation.useLocation)();
  const locationToResolve = (_parentRoute$nextLoca = parentRoute === null || parentRoute === void 0 ? void 0 : parentRoute.nextLocation) !== null && _parentRoute$nextLoca !== void 0 ? _parentRoute$nextLoca : location;
  const resolver = (0, _react.useMemo)(() => _RoutePathResolver.RoutePathResolver.getFromBuilder(builder), [builder]);
  const {
    routeContextValue,
    result
  } = (0, _react.useMemo)(() => {
    const resolved = resolver.resolve(locationToResolve)[0];

    if (resolved === undefined) {
      throw new _LocationNotFoundError.LocationNotFoundError("Current location could not be resolved.");
    }

    const ancestorRoutes = parentRoute ? parentRoute.ancestorRoutes.concat() : [];
    ancestorRoutes.push({
      link: builder.getBuilderLink().getAttachmentRoot(),
      location: (parentRoute === null || parentRoute === void 0 ? void 0 : parentRoute.routeLocation) || {
        pathname: "/",
        state: null
      }
    }); // const result = resolved.route.action(resolved.match as never);

    const result = /*#__PURE__*/_react.default.createElement(resolved.route.action, resolved.match);

    const routeContextValue = {
      route: resolved.route,
      ancestorRoutes,
      routeLocation: resolved.currentLocation,
      nextLocation: resolved.remainingLocation
    };
    return {
      result,
      routeContextValue
    };
  }, [parentRoute, resolver, locationToResolve]);
  return /*#__PURE__*/_react.default.createElement(_RouteContext.RouteContext.Provider, {
    value: routeContextValue
  }, result);
};

exports.useRoutes = useRoutes;